package org.apache.james.mime4j.field.address;

import java.io.StringReader;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import org.apache.james.mime4j.codec.EncoderUtil;
import org.apache.james.mime4j.field.address.Address;
import org.apache.james.mime4j.field.address.Builder;
import org.apache.james.mime4j.field.address.DomainList;
import org.apache.james.mime4j.field.address.parser.AddressListParser;
import org.apache.james.mime4j.field.address.parser.ParseException;

public class Mailbox extends Address {

   private static final long serialVersionUID = 1L;
   private static final DomainList EMPTY_ROUTE_LIST = new DomainList(Collections.emptyList(), true);
   private final String name;
   private final DomainList route;
   private final String localPart;
   private final String domain;


   public Mailbox(String localPart, String domain) {
      this((String)null, (DomainList)null, localPart, domain);
   }

   public Mailbox(DomainList route, String localPart, String domain) {
      this((String)null, route, localPart, domain);
   }

   public Mailbox(String name, String localPart, String domain) {
      this(name, (DomainList)null, localPart, domain);
   }

   public Mailbox(String name, DomainList route, String localPart, String domain) {
      if(localPart != null && localPart.length() != 0) {
         this.name = name != null && name.length() != 0?name:null;
         this.route = route == null?EMPTY_ROUTE_LIST:route;
         this.localPart = localPart;
         this.domain = domain != null && domain.length() != 0?domain:null;
      } else {
         throw new IllegalArgumentException();
      }
   }

   Mailbox(String name, Mailbox baseMailbox) {
      this(name, baseMailbox.getRoute(), baseMailbox.getLocalPart(), baseMailbox.getDomain());
   }

   public static Mailbox parse(String rawMailboxString) {
      AddressListParser parser = new AddressListParser(new StringReader(rawMailboxString));

      try {
         return Builder.getInstance().buildMailbox(parser.parseMailbox());
      } catch (ParseException var3) {
         throw new IllegalArgumentException(var3);
      }
   }

   public String getName() {
      return this.name;
   }

   public DomainList getRoute() {
      return this.route;
   }

   public String getLocalPart() {
      return this.localPart;
   }

   public String getDomain() {
      return this.domain;
   }

   public String getAddress() {
      return this.domain == null?this.localPart:this.localPart + '@' + this.domain;
   }

   public String getDisplayString(boolean includeRoute) {
      includeRoute &= this.route != null;
      boolean includeAngleBrackets = this.name != null || includeRoute;
      StringBuilder sb = new StringBuilder();
      if(this.name != null) {
         sb.append(this.name);
         sb.append(' ');
      }

      if(includeAngleBrackets) {
         sb.append('<');
      }

      if(includeRoute) {
         sb.append(this.route.toRouteString());
         sb.append(':');
      }

      sb.append(this.localPart);
      if(this.domain != null) {
         sb.append('@');
         sb.append(this.domain);
      }

      if(includeAngleBrackets) {
         sb.append('>');
      }

      return sb.toString();
   }

   public String getEncodedString() {
      StringBuilder sb = new StringBuilder();
      if(this.name != null) {
         sb.append(EncoderUtil.encodeAddressDisplayName(this.name));
         sb.append(" <");
      }

      sb.append(EncoderUtil.encodeAddressLocalPart(this.localPart));
      if(this.domain != null) {
         sb.append('@');
         sb.append(this.domain);
      }

      if(this.name != null) {
         sb.append('>');
      }

      return sb.toString();
   }

   public int hashCode() {
      return this.getCanonicalizedAddress().hashCode();
   }

   public boolean equals(Object obj) {
      if(obj == this) {
         return true;
      } else if(!(obj instanceof Mailbox)) {
         return false;
      } else {
         Mailbox other = (Mailbox)obj;
         return this.getCanonicalizedAddress().equals(other.getCanonicalizedAddress());
      }
   }

   protected final void doAddMailboxesTo(List results) {
      results.add(this);
   }

   private Object getCanonicalizedAddress() {
      return this.domain == null?this.localPart:this.localPart + '@' + this.domain.toLowerCase(Locale.US);
   }

}
